/*################### Libraries and constants needed for our SpiroCaptor ###################*/

/*  Luminosity : captor TSL2561  */

#include <Wire.h>
#include "TSL2561.h"

// connect SCL to analog 5
// connect SDA to analog 4
// connect VDD to 3.3V DC
// connect GROUND to common ground
// ADDR can be connected to ground, or vdd or left floating to change the i2c address

// The address will be different depending on whether you let
// the ADDR pin float (addr 0x39), or tie it to ground or vcc. In those cases
// use TSL2561_ADDR_LOW (0x29) or TSL2561_ADDR_HIGH (0x49) respectively
TSL2561 tsl(TSL2561_ADDR_FLOAT);


/*  pH : captor Smecatronix DF028  */

#define SensorPin 1           //pH meter Analog output to Arduino Analog Input 1
#define Offset 0.00            //deviation compensate
unsigned long int avgValuepH;     //Store the average value of the sensor feedback

/*  Intern tempetature : captor DS18B20  */
#include <OneWire.h>


OneWire  ds(10);  // on pin 10 (a 4.7K resistor is necessary)
unsigned long int avgValueT;

/*  Humidity and extern temperature : captor RHT03  */
#include "DHT.h"
DHT dht; 

/*  Turbidity : home made captor  */

int led = 13; // pin where the led is connected

const int luminosityPin = A3; // pin connected to the photoresistance

/*  Radio transmitter : FS1000a  */
#include <VirtualWire.h>
#include "CastDonnees.h"

const int transmit_pin = 12;

  
/*################### SET UP ###################*/
  
void setup() {

/*  Intern tempetature : captor DS18B20  */
  pinMode(13,OUTPUT);
  
Serial.begin(9600);

/*  Luminosity : captor TSL2561  */

  if (tsl.begin()) {
   //Sensor found
  } else {
    //No sensor found
    while (1);
  }
     
  // You can change the gain on the fly, to adapt to brighter/dimmer light situations
  //tsl.setGain(TSL2561_GAIN_0X);         // set no gain (for bright situtations)
  tsl.setGain(TSL2561_GAIN_16X);      // set 16x gain (for dim situations)
  
  // Changing the integration time gives you a longer time over which to sense light
  // longer timelines are slower, but are good in very low light situtations!
  tsl.setTiming(TSL2561_INTEGRATIONTIME_13MS);  // shortest integration time (bright light)
  //tsl.setTiming(TSL2561_INTEGRATIONTIME_101MS);  // medium integration time (medium light)
  //tsl.setTiming(TSL2561_INTEGRATIONTIME_402MS);  // longest integration time (dim light)
  
/*  Humidity and extern temperature : captor RHT03  */

 dht.setup(7); 
 
/*  Radio transmitter : FS1000a  */

  vw_set_tx_pin(transmit_pin);
  vw_setup(2000);	 // Bits per sec
  
/*  Turbidity : home made captor  */  
  pinMode(led, OUTPUT);

}


/*################### LOOP ###################*/

void loop() {
 
/*  Intern tempetature : captor DS18B20  */
byte i;
  byte present = 0;
  byte type_s;
  byte data[12];
  byte addr[8];
  float celsius, fahrenheit;
  
  if ( !ds.search(addr)) {
    ds.reset_search();
    delay(250);
    return;
  }
  

  if (OneWire::crc8(addr, 7) != addr[7]) {
      return;
  }
 
  // the first ROM byte indicates which chip
  switch (addr[0]) {
    case 0x10:
      type_s = 1;
      break;
    case 0x28:
      type_s = 0;
      break;
    case 0x22:
      type_s = 0;
      break;
    default:
      return;
  }

  ds.reset();
  ds.select(addr);
  ds.write(0x44, 1);        // start conversion, with parasite power on at the end
 
  present = ds.reset();
  ds.select(addr);    
  ds.write(0xBE);         // Read Scratchpad

  for ( i = 0; i < 9; i++) {           // we need 9 bytes
    data[i] = ds.read();
  }

  // Convert the data to actual temperature
  // because the result is a 16 bit signed integer, it should
  // be stored to an "int16_t" type, which is always 16 bits
  // even when compiled on a 32 bit processor.
  
    int16_t raw = (data[1] << 8) | data[0];
  if (type_s) {
    raw = raw << 3;                        // 9 bit resolution default
    if (data[7] == 0x10) {                 // "count remain" gives full 12 bit resolution
      raw = (raw & 0xFFF0) + 12 - data[6];
    }
  } else {
    byte cfg = (data[4] & 0x60);           // at lower res, the low bits are undefined, so let's zero them
    if (cfg == 0x00) raw = raw & ~7;       // 9 bit resolution, 93.75 ms
    else if (cfg == 0x20) raw = raw & ~3;  // 10 bit res, 187.5 ms
    else if (cfg == 0x40) raw = raw & ~1;  // 11 bit res, 375 ms
    //// default is 12 bit resolution, 750 ms conversion time
  }
 
  celsius = (float)raw / 16.0;
  fahrenheit = celsius * 1.8 + 32.0;
 
 // return the inter temperature
  Serial.print(celsius);
  Serial.print(" , ");
 
/*  pH : captor Smecatronix DF028  */

 int buf[10];                //buffer for read analog
  for(int i=0;i<10;i++)       //Get 10 sample value from the sensor for smooth the value
  {
    buf[i]=analogRead(SensorPin);
    delay(10);
  }
  for(int i=0;i<9;i++)        //sort the analog from small to large
  {
    for(int j=i+1;j<10;j++)
    {
      if(buf[i]>buf[j])
      {
        int temp=buf[i];
        buf[i]=buf[j];
        buf[j]=temp;
      }
    }
  }
  avgValuepH=0;
  for(int i=2;i<8;i++)                      //take the average value of 6 center sample
    avgValuepH+=buf[i];
    
  float phValue=(float)avgValuepH*5.0/1024/6; //convert the analog into millivolt
 
  phValue=3.5*phValue+Offset;     //convert the millivolt into pH value
 
// Return pH
  Serial.print(phValue,2);
  Serial.print(" , ");
 
  digitalWrite(13, HIGH);      
  delay(800);
  digitalWrite(13, LOW);

/*  Luminosity : captor TSL2561  */
 
  uint16_t x = tsl.getLuminosity(TSL2561_VISIBLE);    
  uint32_t lum = tsl.getFullLuminosity();
  uint16_t ir, full;
  ir = lum >> 16;
  full = lum & 0xFFFF;
  
  
  Serial.print(tsl.calculateLux(full, ir));
  Serial.print(" , ");
  

  /*  Humidity and extern temperature : captor RHT03  */

  float humidity = dht.getHumidity();
  float temperature = dht.getTemperature();

  Serial.print(humidity);
  Serial.print(" , ");
  Serial.print(temperature);
  
  /*  Turbidity : home made captor  */
  
  digitalWrite(led, HIGH);   // turn on the led
  delay(1000);
  
  int j = 0;
  int sensorValue[20];
  int conv[10];
  int sum = 0;
  float photo = 0;
  for(j=0;j<20;j++){
    sensorValue[j] = analogRead(luminosityPin);
    conv[j]=1023-sensorValue[j];
    sum=sum+conv[j];
  }
  photo=sum/20;
  Serial.print(" , ");
  Serial.println(photo);
  digitalWrite(led, LOW);    // turn off the led


   /*  Radio transmitter : FS1000a  */
  Donnees message(celsius,phValue,tsl.calculateLux(full, ir),humidity,temperature,photo);
   char * c = message.toByteArray();
    vw_send((uint8_t *)c, sizeof(Donnees));
  vw_wait_tx();                // Wait until the whole message is gone
    delete c;                  // Release memory


   /////////////////////////////////DELAY////////////////////////////////////////
   
delay (1000); // for quick test  
//delay(3600000);      //   controle le delais des mesures (unite : miliseconde)

}
